@@ -49,6 +49,9 @@ def yamlLoad(stream):
 def yamlDump(data, stream):
   return dumper.dump(data, stream)
 
+def docParser(docstring: str, streamFormat='ini'):
+  from ._funcparse import docParser as _dp
+  return _dp(docstring, streamFormat)
 
 def mproc_apply(func, iterLst, descr='', iterArgPos=0,
                 extraArgs=(), showProgress=True, applyAsync=True,
@@ -190,6 +193,7 @@ class AppLogger(logging.Logger):
     sys.excepthook = self.old_sys_excepthook
 
   def registerWarnings(self):
+    warnings.simplefilter('always', UserWarning)
     self.old_showwarning = warnings.showwarning
     warnings.showwarning = self.warnWithLog
 
@@ -591,30 +595,30 @@ def makeCli(func: t.Callable, convertArgs=True, parserKwargs: dict=None, **procK
   :param parserKwargs: Passed to ArgumentParser on init
   :param procKwargs: Passed to AtomicProcess which is used as an intermediate to generate parameter documentation
   """
-  from .params import NestedProcWrapper
-  from .processing import AtomicProcess
+  from ._funcparse import funcToParamDict, RunOpts
   def tipConverter(inStr):
     doc = QtGui.QTextDocument()
     doc.setHtml(inStr)
     return doc.toPlainText()
 
-  proc = AtomicProcess(func, **procKwargs)
-  param = NestedProcWrapper(proc).parentParam
+  funcDict = funcToParamDict(func, **procKwargs)
+  tip = funcDict.get('func-description', '')
 
   useCls = argparse.ArgumentParser if not convertArgs else ArgParseConverter
   parserKwargs = parserKwargs or {}
-  parserKwargs.update(description=tipConverter(param.opts['tip']))
+  parserKwargs.setdefault('description', tipConverter(tip))
   parser = useCls(**parserKwargs)
 
-  for ch in param:
-    if 'action' not in ch.type():
-      ch: Parameter
-      arg = parser.add_argument(f'--{ch.name()}',
-      type=type(ch.value()))
-      if ch.hasDefault():
-        arg.default = ch.defaultValue()
-      if 'tip' in ch.opts:
-        arg.help = tipConverter(ch.opts['tip'])
+
+  for ch in funcDict['children']:
+    if 'action' in ch['type']:
+      continue
+    arg = parser.add_argument(f'--{ch["name"]}')
+    if ch['value'] is not RunOpts.PARAM_UNSET:
+      arg.type = type(ch['value'])
+      arg.default = ch['value']
+    if 'tip' in ch:
+      arg.help = tipConverter(ch['tip'])
   # if proc.hasKwargs:
   #   sig = inspect.signature(func)
   #   kwParam = [name for name, par in sig.parameters.items() if par.kind is par.VAR_KEYWORD][0]
@@ -645,36 +649,6 @@ def dialogGetSaveFileName(parent, winTitle, defaultTxt: str=None)-> t.Optional[s
   return returnVal
 
 
-def docParser(docstring: str):
-  """
-  From a function docstring, extracts relevant information for how to create smarter
-    parameter boxes.
-
-  :param docstring: Function docstring
-  """
-  parsed = dp.parse(docstring)
-  descrPieces = [p for p in (parsed.short_description, parsed.long_description) if p is not None]
-  descr = ' '.join(descrPieces)
-  out = {}
-  for param in parsed.params:
-    stream = StringIO(param.description)
-    try:
-      paramDoc = yamlLoad(stream)
-    except YAMLError:
-      # Some problem parsing, treat as string
-      paramDoc = param.description
-    if isinstance(paramDoc, str):
-      paramDoc = {'helpText': paramDoc}
-    if paramDoc is None:
-      continue
-    out[param.arg_name] = PrjParam(name=param.arg_name, **paramDoc)
-    if 'pType' not in paramDoc:
-      out[param.arg_name].pType = None
-  # Wrap and extract for proper formatting
-  descr = PrjParam('', helpText=descr).helpText
-  out['top-descr'] = descr
-  return out
-
 def paramWindow(param: Parameter, modal=True):
   tree = flexibleParamTree(param)
   setParamsExpanded(tree, True)
