@@ -3,7 +3,6 @@ import typing as t
 import warnings
 import weakref
 from contextlib import contextmanager
-from enum import Flag, auto
 from pathlib import Path
 
 from pyqtgraph.Qt import QtCore, QtWidgets, QtGui
@@ -12,8 +11,9 @@ from pyqtgraph.parametertree.parameterTypes import GroupParameter
 
 from utilitys.widgets import EasyWidget, DummySignal
 from . import PrjParam
-from .procwrapper import NestedProcWrapper
+from .procwrapper import addStageToParam
 from .. import fns
+from .._funcparse import RunOpts
 from ..processing import *
 from ..typeoverloads import FilePath
 
@@ -24,13 +24,6 @@ __all__ = [
 
 Signal = QtCore.Signal
 
-class RunOpts(Flag):
-  NONE = 0
-  BTN = auto()
-  ON_CHANGED = auto()
-  ON_CHANGING = auto()
-  ON_APPLY = auto()
-
 def _mkRunDict(proc: ProcessStage, btnOpts: t.Union[PrjParam, dict]):
   defaultBtnOpts = dict(name=proc.name, type='shortcut')
   if isinstance(btnOpts, PrjParam):
@@ -41,7 +34,7 @@ def _mkRunDict(proc: ProcessStage, btnOpts: t.Union[PrjParam, dict]):
     btnOpts.pop('type', None)
     defaultBtnOpts.update(btnOpts)
   if len(defaultBtnOpts.get('tip', '')) == 0 and isinstance(proc, AtomicProcess):
-    defaultBtnOpts['tip'] = fns.docParser(proc.func.__doc__)['top-descr']
+    defaultBtnOpts['tip'] = fns.docParser(proc.fnDoc)['func-description']
   if len(proc.input.hyperParamKeys) > 0:
     # In this case, a descriptive name isn't needed since the func name will be
     # present in the parameter group
@@ -49,37 +42,6 @@ def _mkRunDict(proc: ProcessStage, btnOpts: t.Union[PrjParam, dict]):
   return defaultBtnOpts
 
 
-class ParamContainer:
-  """
-  Utility for exposing dict-like behavior to the user for inserting and querying parameter objects. Behaves amost
-  like a pyqtgraph parameter, with the exception that new parameter "children" can be added through the dict
-  interface
-  """
-  def __init__(self):
-    self.params = {}
-
-  def __setitem__(self, key, value):
-    if key not in self.params:
-      # New parameter
-      self.params[key] = value
-    else:
-      self.params[key].setValue(value)
-
-  def __getitem__(self, item):
-    item = self.params[item]
-    try:
-      return item.value()
-    except AttributeError:
-      return item
-
-  def get(self, key, default=None):
-    try:
-      return self[key]
-    except KeyError:
-      return default
-
-  def keys(self):
-    return self.params.keys()
 """
 Eventually, it would be nice to implemenet a global search bar that can find/modify
 any action, shortcut, etc. from any parameter. This tracker is an easy way to fascilitate
@@ -193,7 +155,7 @@ class ParamEditor(QtWidgets.QDockWidget):
       """
       :param loadState:
         pType: popuplineeditor
-        limits: [Default]
+        limits: ['Default']
       """
       if loadState in self.savedStates and loadState != self.stateName:
         return self.loadParamValues(loadState)
@@ -450,7 +412,7 @@ class ParamEditor(QtWidgets.QDockWidget):
 
     return paramAccessor
 
-  def registerFunc(self, func: t.Callable, *, runOpts=RunOpts.BTN,
+  def registerFunc(self, func: t.Callable, *, runOpts=RunOpts.ON_BUTTON,
                    namePath:t.Tuple[str, ...]=(),
                    paramFormat = None,
                    overrideBasePath: t.Sequence[str]=None,
@@ -470,19 +432,17 @@ class ParamEditor(QtWidgets.QDockWidget):
     :param func: Function to make interactive
     :param runOpts: Combination of ways this function can be run. Multiple of these
       options can be selected at the same time using the `|` operator.
-        * If RunOpts.BTN, a button is present as described.
+        * If RunOpts.ON_BUTTON, a button is present as described.
         * If RunOpts.ON_CHANGE, the function is run when parameter values are
           finished being changed by the user
         * If RunOpts.ON_CHANGING, the function is run every time a value is altered,
           even if the value isn't finished changing.
-        * If RunOpts.ON_APPLY, the function is run when "Apply" is pressed. This is useful for when some functions
-          must be run simultaneously.
     :param paramFormat: Formatter which turns variable names into display names. The default takes variables in pascal
       case (e.g. variableName) or snake case (e.g. variable_name) and converts to Title Case (e.g. Variable Name).
       Custom functions must have the signature (str) -> str. To change default behavior, see `nameFormat.set()`.
     :param overrideBasePath: See :meth:`~ParamEditor.registerProp`
     :param btnOpts: Overrides defaults for button used to run this function. If
-      `RunOpts.BTN` is not in `RunOpts`, these values are ignored.
+      `RunOpts.ON_BUTTON` is not in `RunOpts`, these values are ignored.
     :param nest: If *True*, functions with multiple default arguments will have these nested
       inside a group parameter bearing the function name. Otherwise, they will be added
       directly to the parent parameter specified by `namePath` + `baseRegisterPath`
@@ -519,24 +479,21 @@ class ParamEditor(QtWidgets.QDockWidget):
     topParam = fns.getParamChild(parentParam, *namePath)
     if len(proc.input.hyperParamKeys) > 0:
       # Check if proc params already exist from a previous addition
-      wrapped = NestedProcWrapper(proc, topParam, paramFormat or fns.nameFormatter, treatAsAtomic=True, nestHyperparams=nest)
-      parentParam = wrapped.parentParam
+      parentParam = addStageToParam(proc, topParam, argNameFormat=paramFormat, nestHyperparams=nest)
       for param in parentParam:
-        if runOpts & RunOpts.ON_CHANGED:
+        if RunOpts.ON_CHANGED in runOpts:
           param.sigValueChanged.connect(runProc)
-        if runOpts & RunOpts.ON_CHANGING:
+        if RunOpts.ON_CHANGING in runOpts:
           param.sigValueChanging.connect(runpProc_changing)
     else:
       parentParam: GroupParameter = topParam
-    if runOpts & RunOpts.BTN:
+    if RunOpts.ON_BUTTON in runOpts:
       runBtnDict = _mkRunDict(proc, btnOpts)
       if not nest:
         # Make sure button name is correct
         runBtnDict['name'] = proc.name
       runBtn = fns.getParamChild(parentParam, chOpts=runBtnDict)
       runBtn.sigActivated.connect(runProc)
-    if runOpts & RunOpts.ON_APPLY:
-      self.sigChangesApplied.connect(runProc)
     self.procToParamsMapping[proc] = parentParam
 
     if container is not None:
