@@ -6,7 +6,7 @@ from typing import Optional
 
 import numpy as np
 from pyqtgraph.Qt import QtGui, QtWidgets, QtCore
-from pyqtgraph.parametertree import parameterTypes, Parameter
+from pyqtgraph.parametertree import parameterTypes, Parameter, ParameterItem
 from pyqtgraph.parametertree.Parameter import PARAM_TYPES
 from pyqtgraph.parametertree.parameterTypes import ActionParameterItem, ActionParameter, \
   TextParameterItem, TextParameter
@@ -55,7 +55,7 @@ class PgParamDelegate(QtWidgets.QStyledItemDelegate):
     model.setData(index, editor.value())
 
   def setEditorData(self, editor: QtWidgets.QWidget, index):
-    value = index.data(QtCore.Qt.EditRole)
+    value = index.data(QtCore.Qt.ItemDataRole.EditRole)
     editor.setValue(value)
 
   def updateEditorGeometry(self, editor: QtWidgets.QWidget,
@@ -215,9 +215,9 @@ class ProcGroupParameterItem(parameterTypes.GroupParameterItem):
       if proc.allowDisable:
         # Bypass subclass to prevent early short-circuit
         if enabled:
-          super().setData(0, QtCore.Qt.CheckStateRole, QtCore.Qt.Checked)
+          super().setData(0, QtCore.Qt.ItemDataRole.CheckStateRole, QtCore.Qt.CheckState.Checked)
         else:
-          super().setData(0, QtCore.Qt.CheckStateRole, QtCore.Qt.Unchecked)
+          super().setData(0, QtCore.Qt.ItemDataRole.CheckStateRole, QtCore.Qt.CheckState.Unchecked)
       # This gets called before constructor can finish, so add enabled font map here
       proc.disabled = not enabled
       self._mkFontMap()
@@ -229,11 +229,11 @@ class ProcGroupParameterItem(parameterTypes.GroupParameterItem):
     opts = self.param.opts
     flags = self.flags()
     if opts['process'].allowDisable:
-      flags |= QtCore.Qt.ItemIsUserCheckable & (~QtCore.Qt.ItemIsTristate)
+      flags |= QtCore.Qt.ItemFlag.ItemIsUserCheckable & (~QtCore.Qt.ItemFlag.ItemIsTristate)
     self.setFlags(flags)
 
   def setData(self, column, role, value):
-    isChk = role == QtCore.Qt.CheckStateRole
+    isChk = role == QtCore.Qt.ItemDataRole.CheckStateRole
     # Check for invalid request to change check state
     if isChk and self.param.parent() and not self.param.parent().opts['enabled']:
       return
@@ -305,9 +305,9 @@ class SliderParameterItem(parameterTypes.WidgetParameterItem):
 
 
     self.slider = QtWidgets.QSlider()
-    self.slider.setOrientation(QtCore.Qt.Horizontal)
+    self.slider.setOrientation(QtCore.Qt.Orientation.Horizontal)
     lbl = QtWidgets.QLabel()
-    lbl.setAlignment(QtCore.Qt.AlignLeft)
+    lbl.setAlignment(QtCore.Qt.AlignmentFlag.AlignLeft)
 
     w = QtWidgets.QWidget()
     layout = QtWidgets.QHBoxLayout()
@@ -374,71 +374,133 @@ class SliderParameterItem(parameterTypes.WidgetParameterItem):
 class SliderParameter(Parameter):
   itemClass = SliderParameterItem
 
+class ChecklistParameterItem(parameterTypes.GroupParameterItem):
+  def __init__(self, param, depth):
+    self.btnGrp = QtWidgets.QButtonGroup()
+    self.btnGrp.setExclusive(False)
+    self._constructMetaBtns()
+
+    super().__init__(param, depth)
+
+  def _constructMetaBtns(self):
+    self.metaBtnWidget = QtWidgets.QWidget()
+    self.metaBtnLayout = lay = QtWidgets.QHBoxLayout(self.metaBtnWidget)
+    lay.setContentsMargins(0, 0, 0, 0)
+    lay.setSpacing(2)
+    self.metaBtns = {}
+    lay.addStretch(0)
+    for title in 'Clear', 'Select':
+      self.metaBtns[title] = btn = QtWidgets.QPushButton(f'{title} All')
+      self.metaBtnLayout.addWidget(btn)
+      btn.clicked.connect(getattr(self, f'{title.lower()}AllClicked'))
+
+  def treeWidgetChanged(self):
+    ParameterItem.treeWidgetChanged(self)
+    tw = self.treeWidget()
+    if tw is None:
+      return
+    tw.setItemWidget(self, 1, self.metaBtnWidget)
+
+  def selectAllClicked(self):
+    self.param.setValue(list(self.param.names))
+
+  def clearAllClicked(self):
+    self.param.setValue([])
+
+  def insertChild(self, pos, item):
+    ret = super().insertChild(pos, item)
+    self.btnGrp.addButton(item.widget)
+
+  def takeChild(self, i):
+    child = super().takeChild(i)
+    self.btnGrp.removeButton(child.widget)
+
+  def optsChanged(self, param, opts):
+    if 'exclusive' in opts:
+      excl = opts['exclusive']
+      self.btnGrp.setExclusive(excl)
+      for btn in self.metaBtns.values():
+        btn.setDisabled(excl)
+    if 'enabled' in opts:
+      for btn in self.metaBtns.values():
+        btn.setEnabled(opts['expanded'])
+
+  def expandedChangedEvent(self, expanded):
+    for btn in self.metaBtns.values():
+      btn.setVisible(expanded)
+
 class ChecklistParameter(parameterTypes.GroupParameter):
+  itemClass = ChecklistParameterItem
 
   def __init__(self, **opts):
-    opts.setdefault('exclusive', False)
     # Value setting before init causes problems since limits aren't set by that point.
     # Avoid by taking value out of consideration until later
-    value = opts.pop('value', None)
-    super().__init__(**opts)
+    opts.setdefault('limits', [])
+    opts.setdefault('value', opts['limits'])
+    super().__init__(name=opts.pop('name'), exclusive=opts.pop('exclusive', False), value=opts.pop('value'))
+    self.sigLimitsChanged.connect(self.updateLimits)
+    self.sigOptionsChanged.connect(self.optsChanged)
+    self.setLimits(opts.pop('limits', []))
     self.setOpts(**opts)
-    if value is not None:
-      self.setValue(value)
-    self.sigOptionsChanged.connect(lambda obj, opts: self.setOpts(**opts))
 
-  def setLimits(self, limits):
-    super().setLimits(limits)
-    exclusive = self.opts['exclusive']
-    oldOpts = {n: self[n] for n in self.names}
+  def updateLimits(self, _param, limits):
+    oldOpts = self.names
+    val = self.opts['value']
+    # Make sure adding and removing children don't cause tree state changes
+    self.blockTreeChangeSignal()
     self.clearChildren()
+    exclusive = self.opts['exclusive']
     for chOpts in limits:
       if isinstance(chOpts, str):
-        newVal = oldOpts.get(chOpts, not exclusive)
+        # Recycle old values if they match the new limits
+        newVal = oldOpts.get(chOpts, False)
         chOpts = dict(name=chOpts, value=newVal)
       child = self.create(type='bool', **chOpts)
-      self._hookupParam(child, exclusive)
+      # Prevent child from broadcasting tree state changes
       self.addChild(child)
-    if exclusive and len(self.value()) != 1:
-      self.setValue(limits[-1])
-
-  def _hookupParam(self, param, exclusive):
-    def onChange_exclusive(_param, value):
-      if value:
-        self.setValue(param.name())
-    def onChange_default(_param, value):
-      self.sigValueChanged.emit(self, self.value())
-    if exclusive:
-      param.sigValueChanged.connect(onChange_exclusive)
+      child.blockTreeChangeSignal()
+      child.sigValueChanged.connect(self._onSubParamChange)
+    # Purge child changes before unblocking
+    self.treeStateChanges.clear()
+    self.unblockTreeChangeSignal()
+    if exclusive and limits:
+      setVal = val[-1] if val else limits[-1]
     else:
-      param.sigValueChanged.connect(onChange_default)
+      setVal = val
+    if setVal is not None:
+      self.setValue(setVal)
 
-  def setOpts(self, **opts):
-    if 'exclusive' in opts and 'limits' not in opts:
-      self.setLimits(self.opts.get('limits', []))
-    super().setOpts(**opts)
+  def _onSubParamChange(self, param, value):
+    # Old value will become false, new value becomes true. Only fire on new value = true for exclusive
+    if not value and self.opts['exclusive']:
+      return
+    elif self.opts['exclusive']:
+      return self.setValue(param.name())
+    # Interpret value, fire sigValueChanged
+    return self.setValue(self.value())
+
+  def optsChanged(self, param, opts):
+    if 'exclusive' in opts:
+      self.updateLimits(param, self.opts.get('limits', []))
 
   def value(self):
     vals = [p.name() for p in self.children() if p.value()]
-    if not vals or not self.opts['exclusive']:
+    exclusive = self.opts['exclusive']
+    if not vals and exclusive:
+      return None
+    elif exclusive:
+      return vals[0]
+    else:
       return vals
-    return vals[0]
 
   def setValue(self, value, blockSignal=None):
     exclusive = self.opts['exclusive']
     # Will emit at the end, so no problem discarding existing changes
-    if exclusive:
-      value = [value]
-    value = [v for v in value if v in self.names]
-    if value == self.value():
-      return value
-    for chParam in self.childs:
-      chParam.setValue(chParam.name() in value)
-      if exclusive:
-        chParam.setReadonly(chParam.value())
-    newVal = self.value()
-    self.sigValueChanged.emit(self, newVal)
-    return newVal
+    cmpVal = value if isinstance(value, list) else [value]
+    for chParam in self:
+      checked = chParam.name() in cmpVal
+      chParam.setValue(checked, self._onSubParamChange)
+    super().setValue(value, blockSignal)
 
 class NoneParameter(parameterTypes.SimpleParameter):
 
