@@ -53,7 +53,7 @@ class ShortcutsEditor(ParamEditor):
     kwargs.setdefault('namePath', (clsNameOrGroup(overrideOwnerObj),))
     param = self._createSeq(shortcutOpts, **kwargs)
     shc = QtWidgets.QShortcut(overrideOwnerObj)
-    shc.setContext(QtCore.Qt.WidgetWithChildrenShortcut)
+    shc.setContext(QtCore.Qt.ShortcutContext.WidgetWithChildrenShortcut)
 
     shc.activatedAmbiguously.connect(lambda: self.maybeAmbigWarning(param.value()))
 
@@ -112,10 +112,10 @@ class ShortcutsEditor(ParamEditor):
       shortcutParam = PrjParam(shortcutParam)
     matches = [p for p in params_flattened(self.params) if p.name() == shortcutParam['name']]
 
-    formatted = f'<{shortcutParam["name"]}: {shortcutParam["value"]}>'
-    if len(matches) == 0:
-      warnLater(f'Shortcut param {formatted} does not exist. No delete performed.')
-      return
+    # formatted = f'<{shortcutParam["name"]}: {shortcutParam["value"]}>'
+    # if len(matches) == 0:
+    #   warn(f'Shortcut param {formatted} does not exist. No delete performed.', UserWarning)
+    #   return
     for match in matches:
       # Set shortcut key to nothing to prevent its activation
       match.setValue('')
