@@ -1,41 +1,107 @@
 from __future__ import annotations
 
+import copy
 import copy as _copy
 import inspect
+import pickle
 import typing as t
 from abc import ABC
+from collections.abc import MutableMapping
 from functools import wraps
 from warnings import warn
 
 import numpy as np
 import pyqtgraph as pg
 from pyqtgraph.Qt import QtCore
+from pyqtgraph.parametertree import Parameter
+from pyqtgraph.parametertree.Parameter import PARAM_TYPES
 
+from . import misc
+from ._funcparse import funcToParamDict, RunOpts
 from .fns import nameFormatter
 
-__all__ = ['ProcessIO', 'ProcessStage', 'NestedProcess', 'AtomicProcess']
+__all__ = ['ProcessIO', 'ProcessStage', 'NestedProcess', 'AtomicProcess', 'ParamContainer']
+
 
 _infoType = t.List[t.Union[t.List, t.Dict[str, t.Any]]]
 StrList = t.List[str]
 StrCol = t.Collection[str]
 
-class ProcessIO(dict):
+class ParamContainer(MutableMapping):
+  """
+  Utility for exposing dict-like behavior to the user for inserting and querying parameter objects. Behaves amost
+  like a pyqtgraph parameter, with the exception that new parameter "children" can be added through the dict
+  interface
+  """
+  def __init__(self, **kwargs):
+    self.params = {}
+    self.extras = {}
+    for name, value in kwargs.items():
+      self[name] = value
+
+  def __delitem__(self, key):
+    if key in self.extras and key in self.params:
+      raise KeyError(f'Key {key} is ambiguous, exists in both extras and params')
+    try:
+      self.extras.__delitem__(key)
+    except KeyError:
+      # Keyerror on non existence will be raised in the except block
+      self.params.__delitem__(key)
+
+  def __iter__(self):
+    for d in self.extras, self.params:
+      yield from d
+
+  def __len__(self):
+    return self.params.__len__() + self.extras.__len__()
+
+  def __setitem__(self, key, value):
+    if isinstance(value, Parameter):
+      self.params[key] = value
+    elif key in self.params:
+      self.params[key].setValue(value)
+    else:
+      self.extras[key] = value
+
+  def __getitem__(self, item):
+    try:
+      return self.extras[item]
+    except KeyError:
+      return self.params[item].value()
+
+  def copy(self):
+    return copy.copy(self)
+
+  def __deepcopy__(self, memodict):
+    params = self.params
+    # Deepcopy has trouble working with parameters, so temporarily hide them
+    # until after the opre
+    self.params = {}
+    try:
+      deep = pickle.loads(pickle.dumps(self))
+    finally:
+      self.params = params
+    for param, val in params.items():
+      deep.params[param] = Parameter.create(**val.saveState())
+    return deep
+
+class ProcessIO(ParamContainer):
   """
   The object through which the processor pipeline communicates data. Inputs to one process
   become updated by the results from that process, and this updated ProcessIO is used
   as the input to the *next* process.
   """
 
-  class FROM_PREV_IO:
-    """
-    Helper class to indicate whether a key in this IO is supposed to come from
-    a previous process stage. Typical usage:
-    ```if not hyperparam: self[k] = self.FROM_PREV_IO```
+  FROM_PREV_IO = RunOpts.PARAM_UNSET
+  """
+  Helper class to indicate whether a key in this IO is supposed to come from
+  a previous process stage. Typical usage:
+  ```if not hyperparam: self[k] = self.FROM_PREV_IO```
 
-    Typically, process objects will have two IO dictionaries: One that hold the input spec
-    (which makes use of `FROM_PREV_IO`) and one that holds the runtime process values. The
-    latter IO will not make use of `FROM_PREV_IO`.
-    """
+  Typically, process objects will have two IO dictionaries: One that hold the input spec
+  (which makes use of `FROM_PREV_IO`) and one that holds the runtime process values. The
+  latter IO will not make use of `FROM_PREV_IO`.
+  """
 
   def __init__(self, hyperParamKeys: t.List[str]=None, **kwargs) -> None:
     """
@@ -61,7 +127,7 @@ class ProcessIO(dict):
     self.hasKwargs = False
 
   @classmethod
-  def fromFunction(cls, func: t.Callable, ignoreKeys: StrCol=None, **overriddenDefaults):
+  def fromFunction(cls, func: t.Callable, ignoreKeys: StrCol=None, interactive=True, **overriddenDefaults):
     """
     In the ProcessIO scheme, default arguments in a function signature constitute algorithm
     hyperparameters, while required arguments must be provided each time the function is
@@ -70,49 +136,59 @@ class ProcessIO(dict):
     :param func: Function whose input signature should be parsed
     :param ignoreKeys: Keys to disregard entirely from the incoming function. This is useful for cases like adding
       a function at the class instead of instance level and `self` shouldn't be regarded by the parser.
+    :param interactive: Whether pyqtgraph parameters should be created to interact with each hyperparameter
     :param overriddenDefaults: Keys here that match default argument names in `func` will
       override those defaults. If an argument does _not_ have a default in the function definition but should
       be shown to the user, provide a default here and it will appear.
     """
+    # Add custom kwarg to test for **kwarg presence
+    overriddenDefaults['test for kwargs'] = True
+    funcDict = funcToParamDict(func, **overriddenDefaults)
+
+    outDict = {ch['name']: ch for ch in funcDict['children']
+               if not ch.get('ignore', False)}
     if ignoreKeys is None:
       ignoreKeys = []
-    outDict = {}
+    for kk in ignoreKeys:
+      outDict.pop(kk, None)
+
+    values = [ch['value'] for ch in outDict.values()]
+    if not interactive:
+      # Recycle values only
+      outDict = dict(zip(outDict, values))
+
+    hasKwargs = outDict.pop('test for kwargs', False)
     hyperParamKeys = []
-    spec = inspect.signature(func).parameters
-    checkUnclaimedOverrides = False
-    for k, v in spec.items():
-      if (k in ignoreKeys
-              or v.kind is v.VAR_POSITIONAL):
-        continue
-      elif v.kind is v.VAR_KEYWORD:
-        checkUnclaimedOverrides = True
-        continue
-      formattedV = overriddenDefaults.get(k, v.default)
-      if formattedV is v.empty:
-        formattedV = cls.FROM_PREV_IO
-        # Not a hyperparameter
+    for (name, child), val in zip(outDict.items(), values):
+      # Unrepresentable parameters should be turned back into raw values
+      if val is RunOpts.PARAM_UNSET or (interactive and child['type'] not in PARAM_TYPES):
+        outDict[name] = val
       else:
-        hyperParamKeys.append(k)
-      outDict[k] = formattedV
-    # Functions accepting kwargs can be given unnamed arguments from overridden defaults
-    if checkUnclaimedOverrides:
-      unclaimedOverrideKeys = set(overriddenDefaults) - set(hyperParamKeys)
-      for k in unclaimedOverrideKeys:
-        outDict[k] = overriddenDefaults[k]
-        hyperParamKeys.append(k)
-        # TODO: How to determine whether these should be hyperparams as well?
-    # Make sure to avoid 'init got multiple values' error
+        hyperParamKeys.append(name)
+
+    cls._aliasOverlappingArgs(outDict)
+
+    if interactive:
+      for param in hyperParamKeys:
+        outDict[param] = Parameter.create(**outDict[param])
+    out = cls(hyperParamKeys, **outDict)
+    out.hasKwargs = hasKwargs
+    return out
+
+  @classmethod
+  def _aliasOverlappingArgs(cls, paramDict):
+    """
+    Alias arguments in paramDict by prefixing underscores if they conflict with ProcessIO keywords. This allows
+    functions with the same keywords as ProcessIO to still forward them to the inner input dictionary
+    """
     initSig = inspect.signature(cls.__init__).parameters
-    keys = list(outDict)
+    keys = list(paramDict)
     for key in keys:
       if key in initSig:
         newName = '_' + key
-        outDict[newName] = outDict[key]
-        del outDict[key]
-    out = cls(hyperParamKeys, **outDict)
-    if checkUnclaimedOverrides:
-      out.hasKwargs = True
-    return out
+        paramDict[newName] = paramDict[key]
+        del paramDict[key]
+
 
 class ProcessStage(ABC):
   name: str
@@ -267,6 +343,9 @@ class AtomicProcess(ProcessStage):
       self.input.hyperParamKeys = {k for k, v in self.input.items() if v is not None}
     self.hasKwargs = self.input.hasKwargs
     self.result: t.Optional[ProcessIO] = None
+    # If the input changes but no process is run, make sure this doesn't change what the input for the last result
+    # was
+    self.inputForResult: dict = None
     self.fnDoc = (docFunc or func).__doc__
     """Record function documentation of proxy if needed so wrapper functions properly"""
 
@@ -328,14 +407,15 @@ class AtomicProcess(ProcessStage):
 
   def run(self, io: ProcessIO=None, disable=False, **runKwargs):
     disable = disable or self.disabled
-    oldInput = self.input.copy()
     self.updateInput(io, **runKwargs)
-    # Cast to dict since pyqtgraph thinks different types are unequal
     try:
-      cmp = self.cacheOnEq and pg.eq(oldInput, dict(self.input))
+      # Use dicts instead of raw ProcessIO objects since pg params should
+      # be converted to values before comparison
+      cmp = self.cacheOnEq and pg.eq(self.inputForResult, dict(self.input))
     except Exception as ex:
       # Thrown exception is broad on pg equality comparison, so check the message
-      if '==' in str(ex):
+      strEx = str(ex)
+      if '==' in strEx or 'failed to evaluate equivalence' in strEx:
         cmp = False
       else:
         raise
@@ -348,8 +428,9 @@ class AtomicProcess(ProcessStage):
       if self.outMap and not isinstance(out, ProcessIO):
         out = self._maybeWrapStageFunc(out)
       self.result = self.computeArgMapping(self.outMap, out)
+      self.inputForResult = dict(self.input)
     elif not self.cacheOnDisable:
-      self.result = None
+      self.result = self.inputForResult = None
     out = self.result
     if out is None:
       out = self.input
